// SPDX-License-Identifier: GPL-3.0-only
/*
 * Copyright (c) 2008-2023 100askTeam : Dongshan WEI <weidongshan@qq.com> 
 * Discourse:  https://forums.100ask.net
 */
 
/*  Copyright (C) 2008-2023 深圳百问网科技有限公司
 *  All rights reserved
 *
 * 免责声明: 百问网编写的文档, 仅供学员学习使用, 可以转发或引用(请保留作者信息),禁止用于商业用途！
 * 免责声明: 百问网编写的程序, 可以用于商业用途, 但百问网不承担任何后果！
 * 
 * 本程序遵循GPL V3协议, 请遵循协议
 * 百问网学习平台   : https://www.100ask.net
 * 百问网交流社区   : https://forums.100ask.net
 * 百问网官方B站    : https://space.bilibili.com/275908810
 * 本程序所用开发板 : DShanMCU-F103
 * 百问网官方淘宝   : https://100ask.taobao.com
 * 联系我们(E-mail): weidongshan@qq.com
 *
 *          版权所有，盗版必究。
 *  
 * 修改历史     版本号           作者        修改内容
 *-----------------------------------------------------
 * 2024.02.01      v01         百问科技      创建文件
 *-----------------------------------------------------
 */

#ifndef _DRAW_H
#define _DRAW_H

#include <stdint.h>
#include "bitmap.h"
#include "font_chinese.h"
#include "picture.h"


/**********************************************************************
 * 函数名称： Draw_Init
 * 功能描述： Draw初始化,得到LCD的分辨率
 * 输入参数： 无
 * 输出参数： 无
 * 返 回 值： 无
 * 修改日期：      版本号     修改人       修改内容
 * -----------------------------------------------
 * 2024/02/01        V1.0     韦东山       创建
 ***********************************************************************/
void Draw_Init(void);


/**********************************************************************
 * 函数名称： Draw_Clear
 * 功能描述： 把屏幕清屏为某种颜色
 * 输入参数： dwColor, 颜色, 格式为0x00RRGGBB
 * 输出参数： 无
 * 返 回 值： 无
 * 修改日期：      版本号     修改人       修改内容
 * -----------------------------------------------
 * 2024/02/01        V1.0     韦东山       创建
 ***********************************************************************/
void Draw_Clear(uint32_t dwColor);

/**********************************************************************
 * 函数名称： Draw_Region
 * 功能描述： 在指定位置绘制位图
 * 输入参数： x,y - 左上角坐标
 *            ptBitMap - 位图信息,里面含有位图的宽、高、数据
 * 输出参数： 无
 * 返 回 值： 无
 * 修改日期：      版本号     修改人       修改内容
 * -----------------------------------------------
 * 2024/02/01        V1.0     韦东山       创建
 ***********************************************************************/
static void Draw_Region(uint32_t x, uint32_t y, P_BitMap ptBitMap);

/**********************************************************************
 * 函数名称： Draw_Pixel
 * 功能描述： 在指定位置像素颜色
 * 输入参数： x,y - 左上角坐标
 *            dwColor - 颜色, 格式为0x00RRGGBB
 * 输出参数： 无
 * 返 回 值： 无
 * 修改日期：      版本号     修改人       修改内容
 * -----------------------------------------------
 * 2024/02/01        V1.0     韦东山       创建
 ***********************************************************************/
void Draw_Pixel(int x, int y, uint32_t dwColor);

/**********************************************************************
 * 函数名称： Draw_Pixel_In_Buf
 * 功能描述： 把像素颜色先填入buf
 * 输入参数： buf - 存储数据的buf
 *            x,y - 左上角坐标
 *            w,h - buf的分辨率
 *            dwColor - 颜色, 格式为0x00RRGGBB
 * 输出参数： 无
 * 返 回 值： 无
 * 修改日期：      版本号     修改人       修改内容
 * -----------------------------------------------
 * 2024/02/01        V1.0     韦东山       创建
 ***********************************************************************/
void Draw_Pixel_In_Buf(uint8_t *buf, int x, int y, int w, int h, uint32_t dwColor);

/**********************************************************************
 * 函数名称： Draw_ASCII
 * 功能描述： 绘制ASCII字符
 * 输入参数： x,y - 左上角坐标
 *            c   - 字符
 *            front_color - 前景颜色, 格式为0x00RRGGBB
 *            back_color  - 背景颜色, 格式为0x00RRGGBB
 * 输出参数： 无
 * 返 回 值： 字符宽度(单位:像素)
 * 修改日期：      版本号     修改人       修改内容
 * -----------------------------------------------
 * 2024/02/01        V1.0     韦东山       创建
 ***********************************************************************/
int Draw_ASCII(uint32_t x, uint32_t y, char c, uint32_t front_color, uint32_t back_color);

/**********************************************************************
 * 函数名称： Draw_String
 * 功能描述： 绘制ASCII字符串
 * 输入参数： x,y - 左上角坐标
 *            str - 字符串
 *            front_color - 前景颜色, 格式为0x00RRGGBB
 *            back_color  - 背景颜色, 格式为0x00RRGGBB
 * 输出参数： 无
 * 返 回 值： 字符串宽度(单位:像素)
 * 修改日期：      版本号     修改人       修改内容
 * -----------------------------------------------
 * 2024/02/01        V1.0     韦东山       创建
 ***********************************************************************/
int Draw_String(uint32_t x, uint32_t y, char *str, uint32_t front_color, uint32_t back_color);

/**********************************************************************
 * 函数名称： Draw_Line
 * 功能描述： 画线
 * 输入参数： x1,y1 - 起点坐标
 *            x2,y2 - 终点坐标
 *            front_color - 前景颜色, 格式为0x00RRGGBB
 * 输出参数： 无
 * 返 回 值： 无
 * 修改日期：      版本号     修改人       修改内容
 * -----------------------------------------------
 * 2024/02/01        V1.0     韦东山       创建
 ***********************************************************************/
void Draw_Line(uint32_t x1, uint32_t y1, uint32_t x2, uint32_t y2, uint32_t front_color);

/**********************************************************************
 * 函数名称： Draw_Fill
 * 功能描述： 填充矩形区域
 * 输入参数： xsta,ysta - 起点坐标
 *            xend,yend - 终点坐标
 *            dwColor   - 颜色, 格式为0x00RRGGBB
 * 输出参数： 无
 * 返 回 值： 无
 * 修改日期：      版本号     修改人       修改内容
 * -----------------------------------------------
 * 2024/02/01        V1.0     韦东山       创建
 ***********************************************************************/
void Draw_Fill(uint32_t xsta, uint32_t ysta, uint32_t xend, uint32_t yend, uint32_t dwColor);

/**********************************************************************
 * 函数名称： Draw_Sign
 * 功能描述： 画一个十字的标记
 * 输入参数： x,y - 起点坐标
 *            dwColor   - 颜色, 格式为0x00RRGGBB
 * 输出参数： 无
 * 返 回 值： 无
 * 修改日期：      版本号     修改人       修改内容
 * -----------------------------------------------
 * 2024/02/01        V1.0     韦东山       创建
 ***********************************************************************/
void Draw_Sign(uint32_t x, uint32_t y, uint32_t dwColor);

/**********************************************************************
 * 函数名称： Draw_Rectangle
 * 功能描述： 画矩形
 * 输入参数： (x1,y1),(x2,y2):矩形的对角坐标
 *            front_color - 前景颜色, 格式为0x00RRGGBB
 * 输出参数： 无
 * 返 回 值： 无
 * 修改日期：      版本号     修改人       修改内容
 * -----------------------------------------------
 * 2024/02/01        V1.0     韦东山       创建
 ***********************************************************************/
void Draw_Rectangle(uint32_t x1, uint32_t y1, uint32_t x2, uint32_t y2, uint32_t front_color);

/**********************************************************************
 * 函数名称： Draw_Circle
 * 功能描述： 画圆
 * 输入参数： (x0,y0) - 中心点
 *            r       - 半径(单位:像素)
 *            front_color - 前景颜色, 格式为0x00RRGGBB
 * 输出参数： 无
 * 返 回 值： 无
 * 修改日期：      版本号     修改人       修改内容
 * -----------------------------------------------
 * 2024/02/01        V1.0     韦东山       创建
 ***********************************************************************/
void Draw_Circle(uint32_t x0, uint32_t y0, uint32_t r, uint32_t front_color);

/**********************************************************************
 * 函数名称： Draw_Number
 * 功能描述： 以十进制显示数字
 * 输入参数： (x,y) - 坐标
 *            num   - 数值
 *            front_color - 前景颜色, 格式为0x00RRGGBB
 * 输出参数： 无
 * 返 回 值： 显示的字符的总宽度(单位:像素)
 * 修改日期：      版本号     修改人       修改内容
 * -----------------------------------------------
 * 2024/02/01        V1.0     韦东山       创建
 ***********************************************************************/
int Draw_Number(uint32_t x, uint32_t y, uint32_t num, uint32_t front_color);

/**********************************************************************
 * 函数名称： Draw_HexNumber
 * 功能描述： 以16进制显示数字
 * 输入参数： (x,y) - 坐标
 *            num   - 数值
 *            front_color - 前景颜色, 格式为0x00RRGGBB
 * 输出参数： 无
 * 返 回 值： 显示的字符的总宽度(单位:像素)
 * 修改日期：      版本号     修改人       修改内容
 * -----------------------------------------------
 * 2024/02/01        V1.0     韦东山       创建
 ***********************************************************************/
int Draw_HexNumber(uint32_t x, uint32_t y, uint32_t num, uint32_t front_color);

/**********************************************************************
 * 函数名称： Draw_ChineseFont
 * 功能描述： 显示汉字字符串
 * 输入参数： (x,y) - 坐标
 *            cn    - 汉字字符串(UTF-8编码),
 *                    先使用"取字模软件.EXE"生成点阵,存入font_chinese.c的CnChar32x29数组中
 *            front_color - 前景颜色, 格式为0x00RRGGBB
 *            back_color  - 背景颜色, 格式为0x00RRGGBB
 * 输出参数： 无
 * 返 回 值： 无
 * 修改日期：      版本号     修改人       修改内容
 * -----------------------------------------------
 * 2024/02/01        V1.0     韦东山       创建
 ***********************************************************************/
void Draw_ChineseFont(uint32_t x, uint32_t y, char *cn, uint32_t front_color, uint32_t back_color);

/**********************************************************************
 * 函数名称： Draw_Picture
 * 功能描述： 绘制图片
 * 输入参数： (x,y) - 坐标
 *            pic   - 使用image2lcd生成的图片(水平扫描,包含图像头数据,16位真彩色,高位在前)
 *                    先使用"image2lcd.EXE"生成图片的点阵,存入picture.c
 * 输出参数： 无
 * 返 回 值： 无
 * 修改日期：      版本号     修改人       修改内容
 * -----------------------------------------------
 * 2024/02/01        V1.0     韦东山       创建
 ***********************************************************************/
void Draw_Picture(uint32_t x, uint32_t y, const uint8_t *pic);

#endif

