// SPDX-License-Identifier: GPL-3.0-only
/*
 * Copyright (c) 2008-2023 100askTeam : Dongshan WEI <weidongshan@qq.com> 
 * Discourse:  https://forums.100ask.net
 */
 
/*  Copyright (C) 2008-2023 深圳百问网科技有限公司
 *  All rights reserved
 *
 * 免责声明: 百问网编写的文档, 仅供学员学习使用, 可以转发或引用(请保留作者信息),禁止用于商业用途！
 * 免责声明: 百问网编写的程序, 可以用于商业用途, 但百问网不承担任何后果！
 * 
 * 本程序遵循GPL V3协议, 请遵循协议
 * 百问网学习平台   : https://www.100ask.net
 * 百问网交流社区   : https://forums.100ask.net
 * 百问网官方B站    : https://space.bilibili.com/275908810
 * 本程序所用开发板 : DShanMCU-F103
 * 百问网官方淘宝   : https://100ask.taobao.com
 * 联系我们(E-mail): weidongshan@qq.com
 *
 *          版权所有，盗版必究。
 *  
 * 修改历史     版本号           作者        修改内容
 *-----------------------------------------------------
 * 2024.02.01      v01         百问科技      创建文件
 *-----------------------------------------------------
 */
#ifndef __SPI_LCD_H
#define __SPI_LCD_H

#include <stdint.h>

typedef enum
{
    LCD_DISPLAY_ROTATION_0 = 0,
    LCD_DISPLAY_ROTATION_90,
    LCD_DISPLAY_ROTATION_180,
    LCD_DISPLAY_ROTATION_270,
} lcd_display_rotation_t;

/**********************************************************************
 * 函数名称： LCD_WriteDatas
 * 功能描述： 给SPI LCD发送多个数据
 * 输入参数： data  - 数据buf
 *            count - 要发送的数据个数
 * 输出参数： 无
 * 返 回 值： 0 - 成功, (-1) - 错误, (-2) - 忙, (-3) - 超时
 * 修改日期：      版本号     修改人       修改内容
 * -----------------------------------------------
 * 2024/02/01        V1.0     韦东山       创建
 ***********************************************************************/
int LCD_WriteDatas(uint8_t *datas, uint32_t count);

/**********************************************************************
 * 函数名称： LCD_Init
 * 功能描述： 初始化LCD
 * 输入参数： rotation - 旋转角度, 取值如下
 *    LCD_DISPLAY_ROTATION_0,
 *    LCD_DISPLAY_ROTATION_90,
 *    LCD_DISPLAY_ROTATION_180,
 *    LCD_DISPLAY_ROTATION_270,
 * 输出参数： 无
 * 返 回 值： 无
 * 修改日期：      版本号     修改人       修改内容
 * -----------------------------------------------
 * 2024/02/01        V1.0     韦东山       创建
 ***********************************************************************/
void LCD_Init(lcd_display_rotation_t rotation);

/**********************************************************************
 * 函数名称： LCD_SetWindows
 * 功能描述： 设置LCD接收数据的窗口,它会划定一个矩形对应的显存,后面写入的数据都对应这个区域
 * 输入参数： 无
 * 输出参数： (x1,y1) - 左上角坐标(此坐标包含在这个窗口里面)
 *            (x2,y2) - 右下角坐标(此坐标包含在这个窗口里面)
 * 返 回 值： 无
 * 修改日期：      版本号     修改人       修改内容
 * -----------------------------------------------
 * 2024/02/01        V1.0     韦东山       创建
 ***********************************************************************/
void LCD_SetWindows(uint32_t x1, uint32_t y1, uint32_t x2, uint32_t y2);

/**********************************************************************
 * 函数名称： LCD_SetDataLine
 * 功能描述： 设置SPI LCD的D/C引脚为高电平,表示要传输的是数据
 * 输入参数： 无
 * 输出参数： 无
 * 返 回 值： 无
 * 修改日期：      版本号     修改人       修改内容
 * -----------------------------------------------
 * 2024/02/01        V1.0     韦东山       创建
 ***********************************************************************/
void LCD_SetDataLine(void);

/**********************************************************************
 * 函数名称： LCD_GetInfo
 * 功能描述： 获得LCD的信息
 * 输入参数： 无
 * 输出参数： pWidth  - 用来保存LCD的宽度(单位:像素)
 *            pHeight - 用来保存LCD的高度(单位:像素)
 * 返 回 值： 无
 * 修改日期：      版本号     修改人       修改内容
 * -----------------------------------------------
 * 2024/02/01        V1.0     韦东山       创建
 ***********************************************************************/
void LCD_GetInfo(uint32_t *pWidth, uint32_t *pHeight);
    
#endif /* __SPI_LCD_H */

